/*
 * Copyright 2024 KylinSoft Co., Ltd.
 *
 * This program is free software: you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation, either version 3 of the License, or (at your option) any later
 * version.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU General Public License along with
 * this program. If not, see <https://www.gnu.org/licenses/>.
 */

#ifndef SERVICES_TEXTEMBEDDINGPROCESSORPROXY_H
#define SERVICES_TEXTEMBEDDINGPROCESSORPROXY_H

#include <gio/gio.h>

#include <any>
#include <functional>
#include <string>
#include <vector>

#include "coretextembeddingserviceglue.h"
#include "embeddingcommon.h"

class TextEmbeddingProcessorProxy {
public:
    using TextEmbeddingCallback = std::function<void(EmbeddingResult* result)>;
    using TextEmbeddingResultCallback =
        std::function<void(EmbeddingResult* result, void* user_data)>;

    struct EmbeddingRequestContext {
        TextEmbeddingResultCallback callback;
        void* callback_user_data;
    };

public:
    ~TextEmbeddingProcessorProxy();

    int connect();
    int initEngine();
    EmbeddingResult* embeddingText(const std::string& text);

    void embeddingTextAsync(const std::string& text,
                            TextEmbeddingResultCallback callback,
                            void* callback_user_data);
    std::string getModelInfo();

private:
    void destroy();
    static EmbeddingResult parseResult(const std::string& json_string_result);

    static void embeddingTextAsyncCallback(GObject* object, GAsyncResult* res,
                                           gpointer data);

private:
    AiRuntimeCoreTextEmbeddingService* delegate_ = nullptr;
    GDBusConnection* connection_ = nullptr;
    int sessionId_ = -1;
    int initErrorCode_ = -1;
};

#endif